// zur Interaktion zwischen Kamera und Programm:
import processing.video.*;
// für die Verwendung von <<LinkedList>>:
import java.util.*;
// für die Verwendung von <<Math.round()>>:
import java.lang.*;
// für die Datenübertragung zwischen Roboter und Programm:
import oscP5.*;
import netP5.*;

/*   Deklaration von Variablen, die für das Senden 
     und Empfangen von Daten notwendig sind */
OscP5 oscP5;
NetAddress myRemoteLocation;

// Variable, um später ein Bild aufzunehmen
Capture video;

// 2-dim-Array für die Pixel des Bildes
int[][] arr = new int[160][120];

/*  Variable, damit die Printbefehle, 
    die wir zum Testen des Codes an einigen Stellen mit drin haben, 
    in eine Text-Datei geschrieben werden. */
PrintWriter konsole;

/*  Die folgenden Variablen sind jeweils die x/y-Position vom 
    Start- und Zielpunkt, sowie die Größe des aufgenommenen Bildes 
    (deren Werte sich mit der Länge-1 des zuvor deklarierten Arrays) 
    decken müssen. Änderbar, indem man den Start- und Zielpunkt
    jeweils farbig markiert, und anhand einer Farbauswertung die
    Koordinaten bestimmen lässt. *********************************** */
    
final int STARTX=30;   // für das jeweilige Labyrinth manuell anpassen!
final int STARTY=17;

final int ZIELX=53;    // für das jeweilige Labyrinth manuell anpassen!
final int ZIELY=100;

final int SIZEX=159;   // Bildgröße, Kamera-abhängig (hier: 160x120)
final int SIZEY=119;

/*  **************************************************************** */

// Helligkeitsgrenzwert, Skala 0-255, um zwischen hell/dunkel bzw schwarz/weiß zu unterscheiden
int grenzwert_hell = 130; 

// die aktuelle Position des Roboters wird auf die Startkoordinaten gesetzt:
int posX=STARTX;
int posY=STARTY;

/*  Die folgenden Variablen zählen die Schritte, die der 
    Roboter in eine Richtung macht, bevor er sich dreht. 
    Sie werden bei der Wegfindung verwendet und dann via 
    OSC an den Roboter geschickt.  */
int re_s = 0;
int li_s = 0;
int ob_s = 0;
int un_s = 0;

/*  Diese Variablen benötigen wir, um eine korrekte Umrechnung 
    zwischen Bildpixel und tatsächlicher Größe zu erreichen.  */
final double gradrechner = 1.24;   // Wie viele Steps, um X Grad zu erreichen? --> muss steps/90° sein
final double umrechner = 7.14;     // Wie viele Steps, um X Pixel zu fahren?   --> muss steps/pixel sein (Annahme: 1 Pixel = 1 cm)

/*  Liste in der die Befehle, die an den Roboter geschickt 
    werden sollen, gespeichert werden, damit immer ein Befehl 
    nach dem anderen ausgeführt wird.  */
LinkedList<Moving> befehlsspeicher = new LinkedList<Moving>();


void draw() {
  
  /*  gibt die x- und y-Koordinate auf der Konsole aus: z.B. 21 | 3,  */
  konsole.print(posX);
  konsole.print(" | ");
  konsole.print(posY);
  konsole.print(", ");
  
  /*  färbt den Pixel an der aktuellen Stelle ROT  
      (zur Simulation des bereits gefahrenen Wegs)  */
  stroke(255, 0, 0);
  point(posX, posY);
  noStroke();
  
  // Ausgabe: Abstand(Distanz) zum Ziel (wurde im Setup berechnet...)
  konsole.println(arr[posX][posY]);
  
  /*  Wenn das Ziel (= 1) erreicht wurde, wird eine entsprechende 
      Nachricht auf der Konsole ausgegeben, und diese dann danach geschlossen.
      Das delay() ist, um den feritigen Weg (rot) noch etwas betrachten 
      zu können, also damit das Bild nicht sofort geschlossen wird.  
      exit() beendet dann das gesamte Programm. (Hier könnte man dann bspw. 
      auch wieder von vorne das Programm starten lassen)  
      
      Solange das Ziel nicht erreicht ist, wird die wegfindungs-Methode ausgeführt.  */
  if (arr[posX][posY]==1) {
    konsole.println("Am Ziel angekommen!");
    konsole.flush();
    konsole.close();
    delay (60000);
    exit();
  } else {
    wegfindung();
  }
}


/*  Diese Methode schaut, ob der Roboter eine Nachricht gesendet hat 
    (bedeutet, er hat seinen Befehl fertig ausgeführt), und schickt 
    ihm dann den nächsten Befehl aus der Liste.
    
    So wird gewährleistet, dass er alle Befehle auch komplett macht, 
    und nicht sofort abbricht, wenn er den nächsten Befehl erhält, 
    und dann diesen versucht auszuführen.  */
void oscEvent(OscMessage theOscMessage) {
  befehlsspeicher.poll().move();
}